@@ -1,7 +1,7 @@
 package App::lcpan;
 
-our $DATE = '2015-02-26'; # DATE
-our $VERSION = '0.13'; # VERSION
+our $DATE = '2015-04-07'; # DATE
+our $VERSION = '0.14'; # VERSION
 
 use 5.010001;
 use strict;
@@ -36,7 +36,7 @@ our @EXPORT_OK = qw(
 
 our %SPEC;
 
-my %common_args = (
+our %common_args = (
     cpan => {
         schema => 'str*',
         summary => 'Location of your local CPAN mirror, e.g. /path/to/cpan',
@@ -52,6 +52,100 @@ _
         schema  => 'str*',
         default => 'index.db',
         tags => ['common'],
+        completion => sub {
+            my %args = @_;
+            my $word    = $args{word} // '';
+            my $cmdline = $args{cmdline};
+            my $r       = $args{r};
+
+            return undef unless $cmdline;
+
+            # force reading config file
+            $r->{read_config} = 1;
+            my $res = $cmdline->parse_argv($r);
+
+            my $args = $res->[2];
+            _set_args_default($args);
+
+            require Complete::Util;
+            Complete::Util::complete_file(
+                word => $word,
+                starting_path => $args->{cpan},
+                filter => sub {
+                    # file or index.db*
+                    (-d $_[0]) || $_[0] =~ /index\.db/;
+                },
+            );
+        },
+    },
+);
+
+our %query_args = (
+    query => {
+        summary => 'Search query',
+        schema => 'str*',
+        cmdline_aliases => {q=>{}},
+        pos => 0,
+    },
+    detail => {
+        schema => 'bool',
+    },
+);
+
+our %fauthor_args = (
+    author => {
+        summary => 'Filter by author',
+        schema => 'str*',
+        cmdline_aliases => {a=>{}},
+        completion => \&_complete_cpanid,
+    },
+);
+
+our %fdist_args = (
+    dist => {
+        summary => 'Filter by distribution',
+        schema => 'str*',
+        cmdline_aliases => {d=>{}},
+        completion => \&_complete_dist,
+    },
+);
+
+our %flatest_args = (
+    latest => {
+        schema => ['bool*', is=>1],
+    },
+);
+
+our %full_path_args = (
+    full_path => {
+        schema => ['bool*' => is=>1],
+    },
+);
+
+our %mod_args = (
+    module => {
+        schema => 'str*',
+        req => 1,
+        pos => 0,
+        completion => \&_complete_mod,
+    },
+);
+
+our %author_args = (
+    author => {
+        schema => 'str*',
+        req => 1,
+        pos => 0,
+        completion => \&_complete_cpanid,
+    },
+);
+
+our %dist_args = (
+    dist => {
+        schema => 'str*',
+        req => 1,
+        pos => 0,
+        completion => \&_complete_dist,
     },
 );
 
@@ -722,7 +816,16 @@ sub update_local_cpan {
     _set_args_default(\%args);
     my $cpan = $args{cpan};
 
+    my $packages_path = "$cpan/modules/02packages.details.txt.gz";
+    my @st1 = stat($packages_path);
     update_local_cpan_files(%args);
+    my @st2 = stat($packages_path);
+
+    if (@st1 && @st2 && $st1[9] == $st2[9] && $st1[7] == $st2[7]) {
+        $log->infof("%s doesn't change mtime/size, skipping updating index",
+                $packages_path);
+        return [304, "Files did not change, index not updated"];
+    }
     update_local_cpan_index(%args);
 }
 
@@ -892,36 +995,6 @@ sub _complete_cpanid {
     \@res;
 };
 
-my %query_args = (
-    query => {
-        summary => 'Search query',
-        schema => 'str*',
-        cmdline_aliases => {q=>{}},
-        pos => 0,
-    },
-    detail => {
-        schema => 'bool',
-    },
-);
-
-my %fauthor_args = (
-    author => {
-        summary => 'Filter by author',
-        schema => 'str*',
-        cmdline_aliases => {a=>{}},
-        completion => \&_complete_cpanid,
-    },
-);
-
-my %fdist_args = (
-    dist => {
-        summary => 'Filter by distribution',
-        schema => 'str*',
-        cmdline_aliases => {d=>{}},
-        completion => \&_complete_dist,
-    },
-);
-
 $SPEC{list_local_cpan_authors} = {
     v => 1.1,
     summary => 'List authors',
@@ -1061,12 +1134,6 @@ sub list_local_cpan_modules {
     goto &list_local_cpan_packages;
 }
 
-my %flatest_args = (
-    latest => {
-        schema => ['bool*', is=>1],
-    },
-);
-
 $SPEC{list_local_cpan_dists} = {
     v => 1.1,
     summary => 'List distributions',
@@ -1150,12 +1217,6 @@ FROM dist d1".
     \@res;
 }
 
-my %full_path_args = (
-    full_path => {
-        schema => ['bool*' => is=>1],
-    },
-);
-
 $SPEC{'list_local_cpan_releases'} = {
     v => 1.1,
     summary => 'List releases/tarballs',
@@ -1234,216 +1295,6 @@ LEFT JOIN dist d1 ON f1.id=d1.file_id
     \@res;
 }
 
-my %mod_args = (
-    module => {
-        schema => 'str*',
-        req => 1,
-        pos => 0,
-        completion => \&_complete_mod,
-    },
-);
-
-my %author_args = (
-    author => {
-        schema => 'str*',
-        req => 1,
-        pos => 0,
-        completion => \&_complete_cpanid,
-    },
-);
-
-$SPEC{'mod2dist'} = {
-    v => 1.1,
-    summary => 'Get distribution name of a module',
-    args => {
-        %common_args,
-        %mod_args,
-    },
-    result_naked=>1,
-};
-sub mod2dist {
-    my %args = @_;
-
-    _set_args_default(\%args);
-    my $cpan = $args{cpan};
-    my $index_name = $args{index_name};
-    my $mod = $args{module};
-
-    my $dbh = _connect_db('ro', $cpan, $index_name);
-
-    my ($res) = $dbh->selectrow_array("SELECT dist.name
-FROM module
-LEFT JOIN file ON module.file_id=file.id
-LEFT JOIN dist ON file.id=dist.file_id
-WHERE module.name=?", {}, $mod);
-        $res;
-}
-
-$SPEC{'mod2rel'} = {
-    v => 1.1,
-    summary => 'Get (latest) release name of a module',
-    args => {
-        %common_args,
-        %mod_args,
-        %full_path_args,
-        # all=>1
-    },
-    result_naked=>1,
-};
-sub mod2rel {
-    my %args = @_;
-
-    _set_args_default(\%args);
-    my $cpan = $args{cpan};
-    my $index_name = $args{index_name};
-    my $mod = $args{module};
-
-    my $dbh = _connect_db('ro', $cpan, $index_name);
-
-    my $row = $dbh->selectrow_hashref("SELECT
-  file.cpanid cpanid,
-  file.name name
-FROM module
-LEFT JOIN file ON module.file_id=file.id
-WHERE module.name=?
-ORDER BY version_numified DESC
-", {}, $mod);
-    return undef unless $row;
-    if ($args{full_path}) {
-        _relpath($row->{name}, $cpan, $row->{cpanid});
-    } else {
-        $row->{name};
-    }
-}
-
-my %dist_args = (
-    dist => {
-        schema => 'str*',
-        req => 1,
-        pos => 0,
-        completion => \&_complete_dist,
-    },
-);
-
-$SPEC{'dist2rel'} = {
-    v => 1.1,
-    summary => 'Get (latest) release name of a distribution',
-    args => {
-        %common_args,
-        %dist_args,
-        %full_path_args,
-        # all=>1
-    },
-    result_naked=>1,
-};
-sub dist2rel {
-    my %args = @_;
-
-    _set_args_default(\%args);
-    my $cpan = $args{cpan};
-    my $index_name = $args{index_name};
-    my $dist = $args{dist};
-
-    my $dbh = _connect_db('ro', $cpan, $index_name);
-
-    my $row = $dbh->selectrow_hashref("SELECT
-  file.cpanid cpanid,
-  file.name name
-FROM dist
-LEFT JOIN file ON dist.file_id=file.id
-WHERE dist.name=?
-ORDER BY version_numified DESC", {}, $dist);
-    return undef unless $row;
-    if ($args{full_path}) {
-        _relpath($row->{name}, $cpan, $row->{cpanid});
-    } else {
-        $row->{name};
-    }
-}
-
-$SPEC{'distmods'} = {
-    v => 1.1,
-    summary => 'List modules in a distribution',
-    args => {
-        %common_args,
-        %dist_args,
-    },
-    result_naked=>1,
-};
-sub distmods {
-    my %args = @_;
-
-    _set_args_default(\%args);
-    my $cpan = $args{cpan};
-    my $index_name = $args{index_name};
-    my $dist = $args{dist};
-
-    my $dbh = _connect_db('ro', $cpan, $index_name);
-
-    my $sth = $dbh->prepare("SELECT
-  module.name name,
-  module.version version
-FROM module
-LEFT JOIN file ON module.file_id=file.id
-LEFT JOIN dist ON file.id=dist.file_id
-WHERE dist.name=?
-ORDER BY name DESC");
-    $sth->execute($dist);
-    my @res;
-    while (my $row = $sth->fetchrow_hashref) {
-        push @res, $row->{name};
-    }
-    \@res;
-}
-
-$SPEC{'authormods'} = {
-    v => 1.1,
-    summary => 'List modules of an author',
-    args => {
-        %common_args,
-        %author_args,
-    },
-    result_naked=>1,
-};
-sub authormods {
-    my %args = @_;
-
-    list_local_cpan_modules(%args);
-}
-
-$SPEC{'authordists'} = {
-    v => 1.1,
-    summary => 'List distributions of an author',
-    args => {
-        %common_args,
-        %author_args,
-        %flatest_args,
-    },
-    result_naked=>1,
-};
-sub authordists {
-    my %args = @_;
-
-    list_local_cpan_dists(%args);
-}
-
-$SPEC{'authorrels'} = {
-    v => 1.1,
-    summary => 'List releases of an author',
-    args => {
-        %common_args,
-        %author_args,
-        %flatest_args,
-        %full_path_args,
-    },
-    result_naked=>1,
-};
-sub authorrels {
-    my %args = @_;
-
-    list_local_cpan_releases(%args);
-}
-
 sub _get_prereqs {
     require Module::CoreList;
     require Version::Util;
@@ -1722,148 +1573,30 @@ App::lcpan - Manage your local CPAN mirror
 
 =head1 VERSION
 
-This document describes version 0.13 of App::lcpan (from Perl distribution App-lcpan), released on 2015-02-26.
+This document describes version 0.14 of App::lcpan (from Perl distribution App-lcpan), released on 2015-04-07.
 
 =head1 SYNOPSIS
 
 See L<lcpan> script.
 
+=head1 HISTORY
+
+This application began as L<CPAN::SQLite::CPANMeta>, an extension of
+L<CPAN::SQLite>. C<CPAN::SQLite> parses C<02packages.details.txt.gz> and
+C<01mailrc.txt.gz> and puts the parse result into a SQLite database.
+C<CPAN::SQLite::CPANMeta> parses the C<META.json>/C<META.yml> files in
+individual release files and adds it to the SQLite database.
+
+In order to simplify things for the users (one-step indexing) and get more
+freedom in database schema, C<lcpan> skips using C<CPAN::SQLite> and creates its
+own SQLite database. It also parses C<02packages.details.txt.gz> but does not
+parse distribution names from it but instead uses C<META.json> and C<META.yml>
+files extracted from the release files. If no C<META.*> files exist, then it
+will use the module name.
+
 =head1 FUNCTIONS
 
 
-=head2 authordists(%args) -> any
-
-List distributions of an author.
-
-Arguments ('*' denotes required arguments):
-
-=over 4
-
-=item * B<author>* => I<str>
-
-=item * B<cpan> => I<str>
-
-Location of your local CPAN mirror, e.g. /path/to/cpan.
-
-Defaults to C<~/cpan>.
-
-=item * B<index_name> => I<str> (default: "index.db")
-
-Filename of index.
-
-=item * B<latest> => I<bool>
-
-=back
-
-Return value:  (any)
-
-
-=head2 authormods(%args) -> any
-
-List modules of an author.
-
-Arguments ('*' denotes required arguments):
-
-=over 4
-
-=item * B<author>* => I<str>
-
-=item * B<cpan> => I<str>
-
-Location of your local CPAN mirror, e.g. /path/to/cpan.
-
-Defaults to C<~/cpan>.
-
-=item * B<index_name> => I<str> (default: "index.db")
-
-Filename of index.
-
-=back
-
-Return value:  (any)
-
-
-=head2 authorrels(%args) -> any
-
-List releases of an author.
-
-Arguments ('*' denotes required arguments):
-
-=over 4
-
-=item * B<author>* => I<str>
-
-=item * B<cpan> => I<str>
-
-Location of your local CPAN mirror, e.g. /path/to/cpan.
-
-Defaults to C<~/cpan>.
-
-=item * B<full_path> => I<bool>
-
-=item * B<index_name> => I<str> (default: "index.db")
-
-Filename of index.
-
-=item * B<latest> => I<bool>
-
-=back
-
-Return value:  (any)
-
-
-=head2 dist2rel(%args) -> any
-
-Get (latest) release name of a distribution.
-
-Arguments ('*' denotes required arguments):
-
-=over 4
-
-=item * B<cpan> => I<str>
-
-Location of your local CPAN mirror, e.g. /path/to/cpan.
-
-Defaults to C<~/cpan>.
-
-=item * B<dist>* => I<str>
-
-=item * B<full_path> => I<bool>
-
-=item * B<index_name> => I<str> (default: "index.db")
-
-Filename of index.
-
-=back
-
-Return value:  (any)
-
-
-=head2 distmods(%args) -> any
-
-List modules in a distribution.
-
-Arguments ('*' denotes required arguments):
-
-=over 4
-
-=item * B<cpan> => I<str>
-
-Location of your local CPAN mirror, e.g. /path/to/cpan.
-
-Defaults to C<~/cpan>.
-
-=item * B<dist>* => I<str>
-
-=item * B<index_name> => I<str> (default: "index.db")
-
-Filename of index.
-
-=back
-
-Return value:  (any)
-
-
 =head2 list_local_cpan_authors(%args) -> any
 
 List authors.
@@ -1951,7 +1684,7 @@ Recurse for a number of levels (-1 means unlimited).
 
 =item * B<module>* => I<str>
 
-=item * B<perl_version> => I<str> (default: "v5.18.1")
+=item * B<perl_version> => I<str> (default: "v5.18.4")
 
 Set base Perl version for determining core modules.
 
@@ -2207,58 +1940,6 @@ that contains extra information.
 Return value:  (any)
 
 
-=head2 mod2dist(%args) -> any
-
-Get distribution name of a module.
-
-Arguments ('*' denotes required arguments):
-
-=over 4
-
-=item * B<cpan> => I<str>
-
-Location of your local CPAN mirror, e.g. /path/to/cpan.
-
-Defaults to C<~/cpan>.
-
-=item * B<index_name> => I<str> (default: "index.db")
-
-Filename of index.
-
-=item * B<module>* => I<str>
-
-=back
-
-Return value:  (any)
-
-
-=head2 mod2rel(%args) -> any
-
-Get (latest) release name of a module.
-
-Arguments ('*' denotes required arguments):
-
-=over 4
-
-=item * B<cpan> => I<str>
-
-Location of your local CPAN mirror, e.g. /path/to/cpan.
-
-Defaults to C<~/cpan>.
-
-=item * B<full_path> => I<bool>
-
-=item * B<index_name> => I<str> (default: "index.db")
-
-Filename of index.
-
-=item * B<module>* => I<str>
-
-=back
-
-Return value:  (any)
-
-
 =head2 stat_local_cpan(%args) -> [status, msg, result, meta]
 
 Statistics of your local CPAN mirror.
@@ -2419,21 +2100,6 @@ that contains extra information.
 
 Return value:  (any)
 
-=head1 HISTORY
-
-This application began as L<CPAN::SQLite::CPANMeta>, an extension of
-L<CPAN::SQLite>. C<CPAN::SQLite> parses C<02packages.details.txt.gz> and
-C<01mailrc.txt.gz> and puts the parse result into a SQLite database.
-C<CPAN::SQLite::CPANMeta> parses the C<META.json>/C<META.yml> files in
-individual release files and adds it to the SQLite database.
-
-In order to simplify things for the users (one-step indexing) and get more
-freedom in database schema, C<lcpan> skips using C<CPAN::SQLite> and creates its
-own SQLite database. It also parses C<02packages.details.txt.gz> but does not
-parse distribution names from it but instead uses C<META.json> and C<META.yml>
-files extracted from the release files. If no C<META.*> files exist, then it
-will use the module name.
-
 =head1 HOMEPAGE
 
 Please visit the project's homepage at L<https://metacpan.org/release/App-lcpan>.
